@@ -18,18 +18,16 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-css-menu.c,v 1.1.2.5 2004/09/26 16:22:39 chpe Exp $
+ *  $Id: ephy-css-menu.c,v 1.12 2005/04/19 11:30:19 chpe Exp $
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include "ephy-css-menu.h"
 
 #include <epiphany/ephy-window.h>
 #include <epiphany/ephy-embed.h>
-#include "ephy-string.h"
+
 #include "ephy-debug.h"
 
 #include "mozilla/mozilla-helpers.h"
@@ -37,6 +35,8 @@
 #include <gtk/gtkuimanager.h>
 #include <gtk/gtkactiongroup.h>
 #include <gtk/gtkradioaction.h>
+#include <gtk/gtklabel.h>
+#include <gtk/gtkmenuitem.h>
 
 #include <glib/gi18n-lib.h>
 
@@ -63,7 +63,7 @@ struct _EphyCSSMenuPrivate
 #define STYLESHEET_KEY			"ECStyleSheet"
 #define ACTION_VERB_FORMAT		"ECSSSwitchStyle%x"
 #define ACTION_VERB_FORMAT_LENGTH	strlen (ACTION_VERB_FORMAT) + 14 + 1
-#define ACTION_MAX_LENGTH		32
+#define LABEL_WIDTH_CHARS		32
 
 enum
 {
@@ -128,20 +128,16 @@ activate_stylesheet_cb (GtkAction *action,
 	mozilla_set_stylesheet (p->embed, style);
 }
 
-#define MAX_LENGTH 32
-
 static GtkAction *
 create_stylesheet_action (EphyCSSMenu *menu,
 			  MozillaStyleSheet *style,
 			  const char *verb)
 {
 	GtkAction  *action;
-	char *tooltip, *label, *label_temp;
+	char *tooltip;
 	const char *name;
 
 	name = mozilla_stylesheet_get_name (style);
-	label_temp = ephy_string_shorten (name, MAX_LENGTH);
-	label = ephy_string_double_underscores (label_temp);
 
 	switch (mozilla_stylesheet_get_type (style))
 	{
@@ -159,7 +155,7 @@ create_stylesheet_action (EphyCSSMenu *menu,
 
 	action = g_object_new (GTK_TYPE_RADIO_ACTION,
 			       "name", verb,
-			       "label", label,
+			       "label", name,
 			       "tooltip", tooltip,
 			       NULL);
 
@@ -173,13 +169,28 @@ create_stylesheet_action (EphyCSSMenu *menu,
 	gtk_action_group_add_action (menu->priv->item_action_group, action);
 	g_object_unref (action);
 
-	g_free (label_temp);
-	g_free (label);
 	g_free (tooltip);
 
 	return action;
 }
 
+static void
+connect_proxy_cb (GtkActionGroup *action_group,
+                  GtkAction *action,
+                  GtkWidget *proxy)
+{
+	if (GTK_IS_MENU_ITEM (proxy))
+	{
+		GtkLabel *label;
+	
+		label = (GtkLabel *) ((GtkBin *) proxy)->child;
+
+		gtk_label_set_use_underline (label, FALSE);
+		gtk_label_set_ellipsize (label, PANGO_ELLIPSIZE_END);
+		gtk_label_set_max_width_chars (label, LABEL_WIDTH_CHARS);
+	}
+}
+
 static void
 ephy_css_menu_rebuild (EphyCSSMenu *menu)
 {
@@ -212,6 +223,9 @@ ephy_css_menu_rebuild (EphyCSSMenu *menu)
 	/* Create the new action group */
 	p->item_action_group = 
 		gtk_action_group_new ("SelectStylesheetMenuDynamicActions");
+	g_signal_connect (p->item_action_group, "connect-proxy",
+			  G_CALLBACK (connect_proxy_cb), NULL);
+
 	gtk_action_group_set_translation_domain (p->item_action_group, NULL);
 	gtk_ui_manager_insert_action_group (p->manager, p->item_action_group, -1);
 
@@ -219,7 +233,7 @@ ephy_css_menu_rebuild (EphyCSSMenu *menu)
 
 	action = gtk_action_group_get_action (p->menu_action_group,
 					      "ECSSMenuAction");
-	g_object_set (G_OBJECT (action), "sensitive", stylesheets != NULL, NULL);
+	gtk_action_set_sensitive (action, stylesheets != NULL);
 	
 	for (l = stylesheets, i = 0; l != NULL; l = l->next, i++)
 	{
@@ -274,8 +288,6 @@ ephy_css_menu_set_embed (EphyCSSMenu *menu,
 					 G_CALLBACK (ephy_css_menu_rebuild),
 					 menu, G_CONNECT_SWAPPED);
 	}
-
-	ephy_css_menu_rebuild (menu);
 }
 
 static void
@@ -289,9 +301,10 @@ sync_active_tab_cb (EphyWindow *window,
 	g_return_if_fail (embed != NULL);
 
 	ephy_css_menu_set_embed (menu, embed);
+	ephy_css_menu_rebuild (menu);
 }
 
-static GtkActionEntry entries[] =
+static const GtkActionEntry entries[] =
 {
 	{ "ECSSMenuAction", NULL, N_("St_yle"), NULL,
 	  N_("Select a different style for this page"), NULL }
@@ -307,7 +320,7 @@ ephy_css_menu_set_window (EphyCSSMenu *menu, EphyWindow *window)
 	p->window = window;
 	
 	/* Create the Action Group */
-	p->manager = GTK_UI_MANAGER (window->ui_merge);
+	p->manager = GTK_UI_MANAGER (ephy_window_get_ui_manager (window));
 	p->menu_action_group = gtk_action_group_new ("EphyCSSMenuActions");
 	gtk_action_group_set_translation_domain (p->menu_action_group, GETTEXT_PACKAGE);
 	gtk_ui_manager_insert_action_group(p->manager, p->menu_action_group, -1);
@@ -341,6 +354,7 @@ ephy_css_menu_set_window (EphyCSSMenu *menu, EphyWindow *window)
 	{
 		embed = ephy_window_get_active_embed (window);
 		ephy_css_menu_set_embed (menu, embed);
+		ephy_css_menu_rebuild (menu);
 	}
 }
 
@@ -350,6 +364,11 @@ ephy_css_menu_finalize (GObject *object)
 	EphyCSSMenu *menu = EPHY_CSS_MENU(object);
 	EphyCSSMenuPrivate *p = menu->priv;
 
+	g_signal_handlers_disconnect_by_func	
+		(p->window, G_CALLBACK (sync_active_tab_cb), menu);
+
+	ephy_css_menu_set_embed (menu, NULL);
+
 	if (p->ui_id != 0)
 	{
 		gtk_ui_manager_remove_ui (p->manager, p->ui_id);
@@ -409,7 +428,7 @@ ephy_css_menu_set_property (GObject *object,
 	{
 		case PROP_WINDOW:
 			ephy_css_menu_set_window (menu, g_value_get_object (value));
-		break;
+			break;
 	}
 }
 
