@@ -1,7 +1,7 @@
 /*
  *  Copyright (C) 2003 Marco Pesenti Gritti
  *  Copyright (C) 2003 Christian Persch
- *  Copyright (C) 2004 Crispin Flowerday
+ *  Copyright (C) 2004,2005 Crispin Flowerday
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,22 +17,18 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-sidebar-extension.c,v 1.3 2004/10/26 20:42:35 chpe Exp $
+ *  $Id: ephy-sidebar-extension.c,v 1.11 2005/04/19 11:30:19 chpe Exp $
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include "ephy-sidebar-extension.h"
-#include "ephy-sidebar-marshal.h"
 #include "ephy-sidebar-embed.h"
 #include "ephy-sidebar.h"
 
-#include "register-component.h"
-
 #include <epiphany/ephy-extension.h>
 #include <epiphany/ephy-embed-shell.h>
+#include <epiphany/ephy-embed-single.h>
 #include <epiphany/ephy-shell.h>
 #include <epiphany/ephy-session.h>
 #include <epiphany/ephy-node.h>
@@ -103,23 +99,19 @@ typedef struct
 static void cmd_view_sidebar	(GtkAction *action,
 				 EphyWindow *window);
 
-static GtkToggleActionEntry toggle_action_entries [] =
+static const GtkToggleActionEntry toggle_action_entries [] =
 {
 	{ "ViewSidebar", NULL, N_("_Sidebar"), "F9", 
-	  N_("Show or hide the sidebar"), G_CALLBACK(cmd_view_sidebar), FALSE }
+	  N_("Show or hide the sidebar"), G_CALLBACK (cmd_view_sidebar), FALSE }
 };
 
-enum
-{
-	ADD_SIDEBAR,
-	LAST_SIGNAL
-};
-
-static guint signals[LAST_SIGNAL] = { 0 };
-
 static void ephy_sidebar_extension_class_init	(EphySidebarExtensionClass *klass);
 static void ephy_sidebar_extension_iface_init	(EphyExtensionIface *iface);
 static void ephy_sidebar_extension_init		(EphySidebarExtension *extension);
+static void extension_weak_notify_cb		(GObject *dialog,
+						 GObject *extension);
+static void add_dialog_weak_notify_cb		(GObject *extension,
+						 GObject *dialog);
 
 static GObjectClass *parent_class = NULL;
 
@@ -362,9 +354,28 @@ add_dialog_response_cb (GtkDialog *dialog,
 }
 
 static void
-impl_add_sidebar (EphySidebarExtension *extension,
-		  const char *url,
-		  const char *title)
+extension_weak_notify_cb (GObject *dialog,
+			  GObject *extension)
+{
+	g_object_weak_unref (dialog,
+			     (GWeakNotify) add_dialog_weak_notify_cb, extension);
+
+	gtk_widget_destroy (GTK_WIDGET (dialog));
+}
+
+static void
+add_dialog_weak_notify_cb (GObject *extension,
+			   GObject *dialog)
+{
+	g_object_weak_unref (extension,
+			     (GWeakNotify) extension_weak_notify_cb, dialog);
+}
+
+static gboolean
+ephy_sidebar_extension_add_sidebar_cb (EphyEmbedSingle *single,
+                                       const char *url,
+                                       const char *title,
+                                       EphySidebarExtension *extension)
 {
 	EphySession *session;
 	EphyWindow *window;
@@ -381,7 +392,7 @@ impl_add_sidebar (EphySidebarExtension *extension,
 		if (strcmp (node_url, url) == 0)
 		{
 			/* TODO Should we raise a dialog or perform an action here */
-			return;
+			return TRUE;
 		}
 	}
 
@@ -416,7 +427,22 @@ impl_add_sidebar (EphySidebarExtension *extension,
 			       cb_data, (GClosureNotify) free_response_data,
 			       0);
 
+	g_object_weak_ref (G_OBJECT (extension),
+			   (GWeakNotify) extension_weak_notify_cb, dialog);
+	g_object_weak_ref (G_OBJECT (dialog),
+			   (GWeakNotify) add_dialog_weak_notify_cb, extension);
+
 	gtk_widget_show (GTK_WIDGET (dialog));
+
+	return TRUE;
+}
+
+/* work-around for http://bugzilla.gnome.org/show_bug.cgi?id=169116 */
+static void
+fixup (GtkWidget *widget)
+{
+	gtk_widget_hide (widget);
+	gtk_widget_show (widget);
 }
 
 static void
@@ -435,7 +461,7 @@ impl_attach_window (EphyExtension *ext,
 	const char * current_page;
 	gboolean show_sidebar;
 
-	LOG ("EphySidebarExtension attach_window")
+	LOG ("EphySidebarExtension attach_window");
 	
 	manager = GTK_UI_MANAGER (ephy_window_get_ui_manager (window));
 
@@ -485,6 +511,9 @@ impl_attach_window (EphyExtension *ext,
 	gtk_paned_add2 (GTK_PANED(hpaned), notebook);
 	g_object_unref (notebook);
 
+	/* work-around for http://bugzilla.gnome.org/show_bug.cgi?id=169116 */
+	fixup (ephy_window_get_notebook (window));
+
 	gtk_container_add (GTK_CONTAINER (parent), hpaned);
 	gtk_container_child_set_property (GTK_CONTAINER (parent),
 					  hpaned, "position", &position);
@@ -549,7 +578,7 @@ impl_detach_window (EphyExtension *ext,
 	GtkWidget *notebook, *parent;
 	GValue position = {0,};
 
-	LOG ("EphySidebarExtension detach_window")
+	LOG ("EphySidebarExtension detach_window");
 
 	/* Remove UI */
 	manager = GTK_UI_MANAGER (ephy_window_get_ui_manager (window));
@@ -580,6 +609,9 @@ impl_detach_window (EphyExtension *ext,
 	gtk_container_add (GTK_CONTAINER (parent), notebook);
 	g_object_unref (notebook);
 
+	/* work-around for http://bugzilla.gnome.org/show_bug.cgi?id=169116 */
+	fixup (ephy_window_get_notebook (window));
+
 	gtk_container_child_set_property (GTK_CONTAINER (parent),
 					  notebook, "position", &position);
 
@@ -588,10 +620,19 @@ impl_detach_window (EphyExtension *ext,
 	g_object_set_data (G_OBJECT (window), WINDOW_DATA_KEY, NULL);
 }
 
+static void
+impl_fixup_tab (EphyExtension *extension,
+		EphyWindow *window,
+		EphyTab *tab)
+{
+	fixup (GTK_WIDGET (tab));
+}
+		
 static void
 ephy_sidebar_extension_init (EphySidebarExtension *extension)
 {
 	EphyNodeDb *db;
+	GObject *single;
 
 	LOG ("EphySidebarExtension initialising");
 
@@ -609,8 +650,8 @@ ephy_sidebar_extension_init (EphySidebarExtension *extension)
 
 	ephy_node_db_load_from_file (extension->priv->db,
 				     extension->priv->xml_file,
-				     EPHY_SIDEBARS_XML_ROOT,
-				     EPHY_SIDEBARS_XML_VERSION);
+				     (const xmlChar*) EPHY_SIDEBARS_XML_ROOT,
+				     (const xmlChar*) EPHY_SIDEBARS_XML_VERSION);
 	
 	if (ephy_node_get_n_children (extension->priv->state_parent))
 	{
@@ -624,28 +665,28 @@ ephy_sidebar_extension_init (EphySidebarExtension *extension)
 				     extension->priv->state);
 	}
 
-	ephy_embed_shell_get_embed_single (embed_shell); /* Fire up Mozilla */
-
-	mozilla_register_component (G_OBJECT (extension));
+	single = ephy_embed_shell_get_embed_single (embed_shell);
+	g_signal_connect (single, "add-sidebar",
+			  G_CALLBACK (ephy_sidebar_extension_add_sidebar_cb),
+			  extension);
 }
 
 static void
 ephy_sidebar_extension_finalize (GObject *object)
 {
 	EphySidebarExtension *extension = EPHY_SIDEBAR_EXTENSION (object);
+	GObject *single;
 
-	LOG ("EphySidebarExtension finalising")
+	LOG ("EphySidebarExtension finalising");
 
-	mozilla_unregister_component();
-	
 	ephy_node_db_write_to_xml_safe
 		(extension->priv->db, 
-		 extension->priv->xml_file,
-		 EPHY_SIDEBARS_XML_ROOT,
-		 EPHY_SIDEBARS_XML_VERSION,
+		 (const xmlChar*) extension->priv->xml_file,
+		 (const xmlChar*) EPHY_SIDEBARS_XML_ROOT,
+		 (const xmlChar*) EPHY_SIDEBARS_XML_VERSION,
 		 NULL,
-		 extension->priv->sidebars, 0, 
-		 extension->priv->state_parent, 0, 
+		 extension->priv->sidebars, NULL, NULL, 
+		 extension->priv->state_parent, NULL, NULL, 
 		 NULL);
 
 	g_free (extension->priv->xml_file);
@@ -655,6 +696,11 @@ ephy_sidebar_extension_finalize (GObject *object)
 
 	g_object_unref (extension->priv->db);
 
+	/* Disconnect our signal */
+	single = ephy_embed_shell_get_embed_single (embed_shell);
+	g_signal_handlers_disconnect_matched
+		(single, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, object);
+
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -663,6 +709,8 @@ ephy_sidebar_extension_iface_init (EphyExtensionIface *iface)
 {
 	iface->attach_window = impl_attach_window;
 	iface->detach_window = impl_detach_window;
+	iface->attach_tab = impl_fixup_tab;
+	iface->detach_tab = impl_fixup_tab;
 }
 
 static void
@@ -674,19 +722,5 @@ ephy_sidebar_extension_class_init (EphySidebarExtensionClass *klass)
 
 	object_class->finalize = ephy_sidebar_extension_finalize;
 
-	klass->add_sidebar = impl_add_sidebar;
-
-	signals[ADD_SIDEBAR] =
-		g_signal_new ("add_sidebar",
-			      G_OBJECT_CLASS_TYPE (object_class),
-			      G_SIGNAL_RUN_FIRST,
-			      G_STRUCT_OFFSET (EphySidebarExtensionClass, add_sidebar),
-			      NULL, NULL,
-			      ephy_sidebar_marshal_VOID__STRING_STRING,
-			      G_TYPE_NONE,
-			      2,
-			      G_TYPE_STRING,
-			      G_TYPE_STRING);
-
 	g_type_class_add_private (object_class, sizeof (EphySidebarExtensionPrivate));
 }
